# Generated by Django 5.0.12 on 2025-02-14 13:10

import django_fsm
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wbportfolio', '0073_remove_product_price_computation_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='rebalancer',
            name='frequency',
            field=models.CharField(default='RRULE:FREQ=MONTHLY;BYDAY=MO,TU,WE,TH,FR;BYSETPOS=1', help_text='The Evaluation Frequency in RRULE format', max_length=256, verbose_name='Evaluation Frequency'),
        ),
        migrations.AlterField(
            model_name='tradeproposal',
            name='status',
            field=django_fsm.FSMField(choices=[('DRAFT', 'Draft'), ('SUBMIT', 'Submit'), ('APPROVED', 'Approved'), ('DENIED', 'Denied'), ('FAILED', 'Failed')], default='DRAFT', max_length=50, verbose_name='Status'),
        ),
    ]
