# Generated by Django 4.2.6 on 2023-10-31 14:51

from decimal import Decimal

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbportfolio", "0048_alter_trade_status"),
    ]

    operations = [
        migrations.AddField(
            model_name="trade",
            name="claimed_shares",
            field=models.DecimalField(
                decimal_places=4,
                default=Decimal("0"),
                help_text="The number of shares that were claimed.",
                max_digits=15,
                verbose_name="Claimed Shares",
            ),
        ),
    ]
