# Generated by Django 4.1.9 on 2023-07-13 12:00

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("wbportfolio", "0046_add_product_default_account")]

    operations = [
        migrations.AddIndex(
            model_name="fees",
            index=models.Index(fields=["linked_product"], name="wbportfolio_linked__a49c5d_idx"),
        ),
    ]
