# Generated by Django 4.1.10 on 2023-07-26 08:52

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbfdm", "0005_instrument_corporate_action_backend"),
        ("wbportfolio", "0044_alter_instrumentlist_identifier"),
    ]

    operations = [
        migrations.AlterField(
            model_name="instrument",
            name="financial_instrument",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="pms_instrument",
                to="wbfdm.instrument",
                verbose_name="Financial Instrument",
            ),
        ),
    ]
